#ifndef _CHANNEL_H____
#define _CHANNEL_H____

#include "crypto_uint32.h"
#include "iptostr.h"
#include "porttostr.h"
#include "limit.h"

#define CHANNEL_BUFSIZE 131072

struct channel {

    /* channel */
    crypto_uint32 maxpacket;
    crypto_uint32 id;
    crypto_uint32 localwindow;
    crypto_uint32 remotewindow;

    /* child */
    unsigned char buf0[CHANNEL_BUFSIZE];
    long long len0;
    long long pid;
    int fd0;
    int fd1;
    int fd2;
    int status;

    /* ip */
    char localip[IPTOSTR_LEN];
    char localport[PORTTOSTR_LEN];
    char remoteip[IPTOSTR_LEN];
    char remoteport[PORTTOSTR_LEN];

    /* terminal */
    char user[LOGIN_NAME_MAX + 1];
    char termname[TTY_NAME_MAX + 1];
    int flagterminal;
    int master;
    int slave;
    int a;
    int b;
    int x;
    int y;

    /* channel */
    int remoteeof;

};


/* channel_drop.c */
extern int channel_droppriv(struct channel *, char *, char **);
/* channel_fork.c */
extern long long channel_fork(struct channel *, int *);
/* channel_forkpty.c */
extern long long channel_forkpty(struct channel *, int *, int, int);
extern int channel_openpty(struct channel *, int *, int *);

/* channel.c */
extern void channel_purge(struct channel *);
extern int channel_open(struct channel *, const char *, crypto_uint32, crypto_uint32, crypto_uint32, crypto_uint32 *);
extern int channel_openterminal(struct channel *, const char *, crypto_uint32, crypto_uint32, crypto_uint32, crypto_uint32);
extern void channel_ptyresize(struct channel *, crypto_uint32, crypto_uint32, crypto_uint32, crypto_uint32);
extern int channel_env(struct channel *, const char *, const char *);
extern int channel_exec(struct channel *, const char *);
extern void channel_put(struct channel *, unsigned char *, long long);
extern void channel_puteof(struct channel *);
extern int channel_putisready(struct channel *);
extern long long channel_read(struct channel *, unsigned char *, long long);
extern long long channel_extendedread(struct channel *, unsigned char *, long long);
extern int channel_readisready(struct channel *);
extern int channel_extendedreadisready(struct channel *);
extern int channel_write(struct channel *);
extern int channel_writeisready(struct channel *);
extern int channel_iseof(struct channel *);
extern int channel_waitnohang(struct channel *, int *, int *);
extern void channel_purge(struct channel *);
extern void channel_init(struct channel *, );

extern int channel_getfd0(struct channel *);
extern int channel_getfd1(struct channel *);
extern int channel_getfd2(struct channel *);
extern long long channel_getlen0(struct channel *);
extern crypto_uint32 channel_getid(struct channel *);
extern crypto_uint32 channel_getlocalwindow(struct channel *);
extern void channel_incrementremotewindow(struct channel *, crypto_uint32);
extern void channel_incrementlocalwindow(struct channel *, crypto_uint32);

/* channel subsystem.c */
#define CHANNEL_SUBSYSTEM_MAX 64
extern int channel_subsystem_add(struct channel *, const char *);
extern char *channel_subsystem_get(struct channel *, const char *);
extern void channel_subsystem_log(struct channel *);

#endif
